@@ -17,8 +17,6 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using System.Runtime.InteropServices;
-using System.Threading;
-using System.Threading.Tasks;
 using WebKit;
 #if ENABLE_DBUS
 using DBus;
@@ -93,8 +91,8 @@ namespace Imendio.Blam {
         private TrayIcon    trayIcon;
 
         public static string BaseDir;
-        public static SynchronizationContext Context { get; private set; }
 
+        private AddChannelDialog  addChannelDialog;
         private ChannelDialog     channelDialog;
         private AddGroupDialog    addGroupDialog;
         private PreferencesDialog preferencesDialog;
@@ -102,7 +100,7 @@ namespace Imendio.Blam {
 
         private ChannelCollection mCollection;
 
-        Timer                     AutoRefreshTimer;
+        private uint              mAutoRefreshId;
 
         public static TargetEntry[] DragEntries = new TargetEntry[] {
             new TargetEntry("STRING", 0, (uint)TargetType.String),
@@ -145,8 +143,6 @@ namespace Imendio.Blam {
             SetupDBus();
 
             Gtk.Application.Init ();
-			Context = new GLib.GLibSynchronizationContext();
-			SynchronizationContext.SetSynchronizationContext(Context);
 
             Proxy.InitProxy ();
 
@@ -156,9 +152,12 @@ namespace Imendio.Blam {
                 Directory.CreateDirectory(Defines.APP_HOMEDIR);
             }
 
-            ItemStore.Load();
             mCollection = ChannelCollection.LoadFromFile (Defines.APP_HOMEDIR + "/collection.xml");
 
+            mCollection.ChannelUpdated         += ChannelUpdatedCb;
+            mCollection.ChannelAdded           += ChannelAddedCb;
+			mCollection.ChannelGroupAdded      += ChannelGroupAddedCb;
+            mCollection.ChannelRemoved         += ChannelRemovedCb;
             mCollection.ChannelRefreshStarted  += ChannelRefreshStartedCb;
             mCollection.ChannelRefreshFinished += ChannelRefreshFinishedCb;
 
@@ -184,7 +183,7 @@ namespace Imendio.Blam {
                                                         "mainWindow", null);
             gladeXML.Autoconnect(this);
 
-            channelList = new ChannelList(mCollection);
+            channelList = new ChannelList(mCollection.Channels, mCollection.Groups);
             ((Container)channelListSw).Child = channelList;
 
             channelList.ChannelSelectedEvent   += ChannelSelected;
@@ -193,10 +192,7 @@ namespace Imendio.Blam {
             channelList.RemoveChannelEvent     += RemoveChannelActivated;
             channelList.RefreshChannelEvent    += RefreshChannelActivated;
 
-			itemList = new ItemList(itemView, channelList);
-			((Container)itemListSw).Child = itemList;
-
-            itemView = new ItemView(itemList);
+            itemView = new ItemView ();
 
             Frame f = new Frame ();
             f.Shadow = ShadowType.In;
@@ -204,11 +200,14 @@ namespace Imendio.Blam {
             itemPaned.Add2 (f);
             f.Show ();
             itemView.OnUrl += OnUrl;
+            //itemView.Widget.DomMouseClick += new Gecko.DomMouseClickHandler(OnButtonPressEvent);
 
-			// a bit silly to do it every time, but works
-			itemList.PropertyChanged += (sender, e) => printMenuItem.Sensitive = true;
+            itemList = new ItemList(itemView);
+            ((Container)itemListSw).Child = itemList;
 
-            trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"), mCollection);
+            itemList.ItemSelected += ItemSelected;
+
+            trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"));
             trayIcon.ButtonPressEvent += TrayIconButtonPressCb;
             trayIcon.RefreshAllEvent += RefreshAllActivated;
             trayIcon.PreferencesEvent += PreferencesActivated;
@@ -240,10 +239,11 @@ namespace Imendio.Blam {
             }
 
             channelDialog = new ChannelDialog (this);
+            addChannelDialog = new AddChannelDialog (this);
             addGroupDialog = new AddGroupDialog (this);
             preferencesDialog = new PreferencesDialog (this.Window);
             opmlDialog = new OpmlDialog (this.Window);
-            opmlDialog.ChannelAdded += channel => mCollection.Add(channel, true);
+            opmlDialog.ChannelAdded += mCollection.Add;
             opmlDialog.ImportFinished += OpmlImportFinished;
 
         }
@@ -333,6 +333,8 @@ namespace Imendio.Blam {
                 return;
             }
 
+            itemList.CurrentChannel = channel;
+
             blogNameLabel.Markup = "<b>" + channel.Name + "</b>";
             mainWindow.Title = "Blam - " + channel.Name;
 
@@ -348,6 +350,12 @@ namespace Imendio.Blam {
             editEntryKeywordsMenuItem.Sensitive = sensitive;
         }
 
+        private void ItemSelected(Imendio.Blam.Item item)
+        {
+            itemView.CurrentItem = item;
+            printMenuItem.Sensitive = true;
+        }
+
         public void MarkEntryAsUnreadActivated (object obj, EventArgs args)
         {
             Item item = itemList.GetSelected ();
@@ -356,8 +364,7 @@ namespace Imendio.Blam {
             }
 
             // Toggle unread status
-            item.Unread = !item.Unread;
-            itemList.EmitSelectedRowChanged();
+            item.SetUnread (!item.Unread);
         }
 
         public void EditEntryKeywordsActivated (object obj, EventArgs args)
@@ -406,7 +413,7 @@ namespace Imendio.Blam {
         public void NextUnreadActivated(object obj, EventArgs args)
         {
             if (channelList.NextUnread()) {
-                itemList.Next(true);
+                itemList.NextUnread();
             }
         }
 
@@ -418,10 +425,6 @@ namespace Imendio.Blam {
                 // Couldn't figure out how to get this to the menu item itself.
                 nextUnreadMenuItem.Activate ();
                 break;
-                case Gdk.Key.j:
-                case Gdk.Key.J:
-                    itemList.Next(false);
-                    break;
             case (Gdk.Key.Escape):
                 mainWindow.Hide ();
                 trayIcon.Show();
@@ -536,6 +539,7 @@ namespace Imendio.Blam {
             mainWindow.Hide();
             mCollection.SaveToFile ();
             ItemStore.Save();
+            mCollection.StopAllThreads();
 
             Gtk.Main.Quit();
         }
@@ -552,33 +556,12 @@ namespace Imendio.Blam {
 
         public void AboutActivated (object obj, EventArgs args)
         {
-            var about = new Gtk.AboutDialog();
-            about.TransientFor = TheApp.mainWindow;
-            about.Version = Defines.VERSION + "\n" + Defines.CODENAME;
-            about.LogoIconName = "blam";
-            about.IconName = "blam";
-            about.Copyright = "Copyright 2004-2006 (c) Mikael Hallendal <micke@imendio.com>\n"
-                + "Copyright 2006-20012 (c) Carlos Martín Nieto <cmn@dwim.me>";
-            about.Run();
-            about.Destroy();
+            new AboutDialog(mainWindow);
         }
 
-        public void AddChannelActivated(object obj, EventArgs args)
+        public void AddChannelActivated (object obj, EventArgs args)
         {
-            using (var diag = new AddChannelDialog(TheApp.Window)) {
-                var response = diag.Run();
-
-                diag.Hide();
-                switch (response) {
-                case (int)ResponseType.Ok:
-                    var chan = new Channel();
-                    chan.Url = diag.Url;
-                    chan.http_username = diag.Username;
-                    chan.http_password = diag.Password;
-                    TheApp.CCollection.Add(chan, true);
-                    break;
-                }
-			}
+            addChannelDialog.Show ();
         }
 
         public void AddGroupActivated (object obj, EventArgs args)
@@ -643,7 +626,7 @@ namespace Imendio.Blam {
         public void RefreshChannelActivated(IChannel channel)
         {
             if (channel != null) {
-                channel.RefreshAsync();
+                mCollection.Refresh (channel);
             }
         }
 
@@ -659,7 +642,7 @@ namespace Imendio.Blam {
             /* First move the refresh back */
             StartStopAutoRefresh();
             /* And pretend a timeout occurred */
-            DoRefreshAll();
+            TimeoutRefreshAll();
         }
 
         private void DragDataReceivedCb(object o, DragDataReceivedArgs args)
@@ -676,8 +659,7 @@ namespace Imendio.Blam {
             UTF8Encoding encoding = new UTF8Encoding( );
             string text = encoding.GetString(d.Data);
 
-            var diag = new AddChannelDialog(TheApp.Window);
-            diag.Url = text;
+            addChannelDialog.Show (text);
         }
 
         private void OnUrl (string url)
@@ -690,6 +672,28 @@ namespace Imendio.Blam {
             }
         }
 
+        private void ChannelAddedCb (IChannel channel)
+        {
+            channelList.Add (channel);
+        }
+
+		private void ChannelGroupAddedCb(IChannel group, IChannel chan)
+		{
+			channelList.AddToGroup(group, chan);
+		}
+
+        private void ChannelUpdatedCb (IChannel channel)
+        {
+            channelList.Updated (channel);
+            UpdateTotalNumberOfUnread ();
+        }
+
+        private void ChannelRemovedCb (IChannel channel)
+        {
+            channelList.Remove (channel);
+            UpdateTotalNumberOfUnread ();
+        }
+
         private void ChannelRefreshStartedCb (IChannel channel)
         {
             uint contextId = statusbar.GetContextId("update-status");
@@ -705,10 +709,11 @@ namespace Imendio.Blam {
             uint contextId = statusbar.GetContextId("update-status");
 
             statusbar.Pop (contextId);
+            channelList.Updated (channel);
             UpdateTotalNumberOfUnread ();
 
             if (channelList.GetSelected () == channel) {
-                itemList.UpdateList();
+                itemList.UpdateList ();
             }
         }
 
@@ -727,25 +732,33 @@ namespace Imendio.Blam {
             statusbar.Push(contextId, StatusString);
         }
 
-		void DoRefreshAll(object obj = null)
+        private bool TimeoutRefreshAll ()
         {
             ShowNextUpdateTime();
             mCollection.RefreshAll();
+
+            /* Continue until source is removed */
+            return true;
         }
 
-		private void StartStopAutoRefresh ()
-		{
-			bool doAutoRefresh = Conf.Get (Preference.AUTO_REFRESH, false);
-			if (!doAutoRefresh)
-				return;
+        private void StartStopAutoRefresh ()
+        {
+            bool doAutoRefresh;
 
-			var rate = TimeSpan.FromMinutes(Conf.Get(Preference.AUTO_REFRESH_RATE, 15));
-			if (AutoRefreshTimer != null) {
-				AutoRefreshTimer.Change(rate, rate);
-			}
+            if (mAutoRefreshId != 0) {
+                GLib.Source.Remove (mAutoRefreshId);
+                mAutoRefreshId = 0;
+            }
 
-			AutoRefreshTimer = new Timer(DoRefreshAll, null, rate, rate);
-		}
+            doAutoRefresh = Conf.Get (Preference.AUTO_REFRESH, false);
+            if (!doAutoRefresh) {
+                return;
+            }
+
+            // Check if any feeds require refresh every ten seconds
+            mAutoRefreshId = GLib.Timeout.Add ((uint) Conf.Get(Preference.AUTO_REFRESH_RATE, 15) * 60 * 1000,
+                                               new GLib.TimeoutHandler (TimeoutRefreshAll));
+        }
 
         private void ConfNotifyHandler (object sender, NotifyEventArgs args)
         {
@@ -821,7 +834,7 @@ namespace Imendio.Blam {
             Conf.Sync ();
         }
 
-        public void UpdateTotalNumberOfUnread()
+        private void UpdateTotalNumberOfUnread ()
         {
             int nrOfUnread, nrOfNew;
 
@@ -832,6 +845,14 @@ namespace Imendio.Blam {
                                                  channelsLabelText +
                                                  "</b>",
                                                  nrOfUnread);
+
+            /* Total number of unread items */
+            string str = string.Format (Catalog.GetPluralString ("{0} unread item", "{0} unread items", nrOfUnread),
+                                             nrOfUnread);
+            str += " ";
+            /* Number of new (not-skipped-over) entries. Gets appended to previous string */
+            str += string.Format(Catalog.GetPluralString("({0} new)", "({0} new)", nrOfNew), nrOfNew);
+            trayIcon.Tooltip = str;
         }
 
 	public static void SetProcessName(string name)
