@@ -6,17 +6,9 @@
 
 using System.Collections;
 using System;
-using System.Linq;
 using System.Net;
-using System.Net.Security;
-using System.Xml;
 using System.Xml.Serialization;
-using System.IO;
-using System.Threading.Tasks;
-using System.ComponentModel;
 using System.ServiceModel.Syndication;
-using System.Security.Cryptography.X509Certificates;
-using System.Runtime.CompilerServices;
 
 namespace Imendio.Blam {
 
@@ -31,12 +23,14 @@ namespace Imendio.Blam {
         bool MarkAsRead();
         Item GetItem(string id);
         void Setup();
-        event PropertyChangedEventHandler PropertyChanged;
+        event ChannelEventHandler Updated;
         void RemoveItems();
-		Task<bool> RefreshAsync();
     }
 
-    public class Channel : IChannel, INotifyPropertyChanged {
+    public class Channel : IChannel {
+		[XmlAttribute("Name")] public string Int_Name = "";
+		[XmlAttribute("Url")] public string Int_Url = "";
+
 		// Used when updating the feed
 		[XmlAttribute] public string LastModified = "";
 		[XmlAttribute] public string ETag = "";
@@ -48,41 +42,32 @@ namespace Imendio.Blam {
 		[XmlAttribute] public string http_username = "";
 		[XmlAttribute] public string http_password = "";
 
+        ItemStore store = null;
         ArrayList item_list = null;
         Object obj = new Object();
 
-		public event PropertyChangedEventHandler PropertyChanged;
-
-		string name;
-		[XmlAttribute]
-		public string Name {
-			get {
-				return name;
-			}
-			set {
-				if (name != value) {
-					name = value;
-					NotifyPropertyChanged();
-				}
-			}
-		}
-
-		string url;
-		[XmlAttribute]
-		public string Url {
-			get {
-				return url;
-			}
-			set {
-				if (url != value) {
-					url = value;
-					NotifyPropertyChanged();
-				}
-			}
-		}
+        public event ChannelEventHandler Updated;
 
         private Gtk.TreeIter mIter;
 
+        public string Name {
+            get {
+                return Int_Name;
+            }
+            set {
+                Int_Name = value;
+            }
+        }
+
+        public string Url {
+            get {
+                return Int_Url;
+            }
+            set {
+                Int_Url = value;
+            }
+        }
+
 		public int NrOfItems {
 			get {
                 lock(obj){
@@ -99,7 +84,7 @@ namespace Imendio.Blam {
                 lock(obj){
                     foreach (string id in item_list) {
                         if(id != null){
-                            item = ItemStore.Get(id);
+                            item = store.Get(id);
                             if(item.Unread){
                                 ++unread;
                             }
@@ -120,7 +105,7 @@ namespace Imendio.Blam {
                         if(id == null){ /* TODO: Figure out why this happens */
                             continue;
                         }
-                        item = ItemStore.Get(id);
+                        item = store.Get(id);
                         if(item != null && item.Unread && !item.Old){
                             ++new_items;
                         }
@@ -145,13 +130,6 @@ namespace Imendio.Blam {
             }
         }
 
-		void NotifyPropertyChanged([CallerMemberName] string name = "")
-		{
-			if (PropertyChanged != null) {
-				PropertyChanged(this, new PropertyChangedEventArgs(name));
-			}
-		}
-
 		[XmlIgnore]
         public Gtk.TreeIter Iter {
             get {
@@ -166,6 +144,7 @@ namespace Imendio.Blam {
         {
             item_list = new ArrayList();
             mIter = new Gtk.TreeIter();
+            store = ItemStore.GetInstance();
         }
 
         public Channel (string name, string url) : this()
@@ -176,7 +155,7 @@ namespace Imendio.Blam {
 
         public Item GetItem (string id)
         {
-            return ItemStore.Get(id);
+            return store.Get(id);
         }
 
         public void Setup()
@@ -187,11 +166,11 @@ namespace Imendio.Blam {
                     if(id == null){
                         continue;
                     }
-                    Item item = ItemStore.Get(id);
+                    Item item = store.Get(id);
                     if(item == null){
                     } else {
                         nlist.Add(id);
-                        item.PropertyChanged += ItemChanged;
+                        item.Updated += ItemUpdated;
                     }
                 }
                 item_list = nlist;
@@ -200,9 +179,10 @@ namespace Imendio.Blam {
 
 		public void RemoveItems()
 		{
+			ItemStore store = ItemStore.GetInstance();
 			lock(obj){
 				foreach(string id in item_list){
-					ItemStore.Remove(id);
+					store.Remove(id);
 				}
 			}
 		}
@@ -210,74 +190,81 @@ namespace Imendio.Blam {
 		public bool MarkAsRead ()
 		{
 			bool updated = false;
-			Item item;
+            Item item;
 
-			lock (obj) {
-				foreach (string id in item_list) {
-					if (id == null) {
-						System.Console.WriteLine("null id {0} on {1}", id, Url);
-						continue;
-					}
-					item = ItemStore.Get(id);
-					if (item.Unread) {
-						item.Unread = false;
-						updated = true;
-					}
-				}
-			}
-
-			if (updated)
-				NotifyPropertyChanged("NrOfUnreadItems");
+            lock(obj){
+                foreach(string id in item_list){
+                    if(id == null){
+                        System.Console.WriteLine("null id {0} on {1}", id, Url);
+                        continue;
+                    }
+                    item = store.Get(id);
+                    if (item.Unread) {
+                        item.SetUnread (false);
+                        updated = true;
+                        EmitUpdated();
+                    }
+                }
+            }
 
 			return updated;
 		}
 
+        public void EmitUpdated()
+        {
+            if(Updated != null){
+                Updated(this);
+            }
+        }
+
         /**
          * Update this channel with the given feed
          */
-		public bool Update(SyndicationFeed feed)
-		{
-			/* If our name is sub-optimal */
-			if((Name == "" || Name == Url) &&
-			   feed.Title.Text != null) {
-				Name = HtmlUtils.StripHtml(feed.Title.Text.Trim());
-			}
+        public bool Update(SyndicationFeed feed)
+        {
+            lock(obj){
+                /* If our name is sub-optimal */
+                if((Name == "" || Name == Url) &&
+                   feed.Title.Text != null) {
+                    Name = HtmlUtils.StripHtml(feed.Title.Text.Trim());
+                }
 
-			LastRefreshed = DateTime.Now;
+                LastRefreshed = DateTime.Now;
+                /*
+                 * The new item list is just the new list, so create it
+                 * on the spot
+                 */
+                ArrayList nitems = new ArrayList();
+                foreach(SyndicationItem sitem in feed.Items){
+					nitems.Add(sitem.Id);
+                    /* If didn't have it already, add it to the store */
+                    if(!item_list.Contains(sitem.Id)) {
+                        store.Add(new Item(sitem));
+					} else {
+						if (sitem.Id == null)
+							continue;
+						
+						store.Get(sitem.Id).Update(sitem);
+					}
+                }
 
-			lock (obj ){
-				var common = feed.Items.Where(i => item_list.Contains(i.Id));
-				foreach (var item in common)
-					ItemStore.Get(item.Id).Update(item);
+                /* Delete the items that are no longer in the feed */
+                foreach(string id in item_list){
+                    if(!nitems.Contains(id)){
+                        store.Remove(id);
+                    }
+                }
 
-				// new items
-				foreach (var item in feed.Items.Except(common))
-					ItemStore.Add(new Item(item));
+                item_list = nitems;
+            }
 
-				// items no longer in the feed are evicted
-				var ids = feed.Items.Select(i => i.Id);
-				var evict = item_list.Cast<string>().Except(ids);
-				foreach (var id in evict)
-					ItemStore.Remove(id);
+            return true;
+        }
 
-				item_list = new ArrayList(ids.ToArray());
-			}
-
-			NotifyPropertyChanged("NrOfUnreadItems");
-			return true;
-		}
-
-		void ItemChanged(object sender, PropertyChangedEventArgs args)
-		{
-			switch (args.PropertyName) {
-				case "Unread":
-					NotifyPropertyChanged("NrOfUnreadItems");
-					break;
-				default:
-					throw new InvalidOperationException("Item changed in unexpected way. Shouldn't happen");
-					break;
-			}
-		}
+        public void ItemUpdated(Item item)
+        {
+            EmitUpdated();
+        }
 
         /**
          * Adds or updates the entry
@@ -286,15 +273,25 @@ namespace Imendio.Blam {
         {
             if(item_list.Contains(item.Id)){
                 /* In this case we only need to update */
-                ItemStore.Get(item.Id).Update(item);
+                store.Get(item.Id).Update(item);
                 return;
             }
 
-			ItemStore.Add(item);
+            store.Add(item);
             lock(obj){
                 item_list.Add(item.Id);
             }
-			ItemStore.Get(item.Id).PropertyChanged += ItemChanged;
+            store.Get(item.Id).Updated += ItemUpdated;
+        }
+
+        /* Used to cross-mark as read */
+        public void MarkItemIdAsRead (string id)
+        {
+            Item item = store.Get(id);
+            if (item.Unread) {
+                item.Unread = false;
+                EmitUpdated();
+            }
         }
 
 		public bool GetHasKeyword (string keyword)
@@ -305,31 +302,7 @@ namespace Imendio.Blam {
 
 			return false;
 		}
-
-		bool ValidateCertificate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
-		{
-			return Conf.Get(Preference.IGNORE_SSL_ERR, false);
-		}
-
-		SyndicationFeed LoadFeed(string url)
-		{
-			var opts = new XmlReaderSettings();
-			opts.CheckCharacters = false;
-			var reader = XmlReader.Create(url, opts);
-			return SyndicationFeed.Load(reader);
-		}
-
-		public async Task<bool> RefreshAsync()
-		{
-			ServicePointManager.ServerCertificateValidationCallback = ValidateCertificate;
-			try {
-				var feed = await Task.Run(() => LoadFeed(Url));
-
-				return Update(feed);
-			} catch (Exception e) {
-				Console.WriteLine("Failed to get feed {0}: {1}", Name, e.Message);
-				return false;
-			}
-		}
 	}
+
+
 }
