@@ -7,22 +7,18 @@
 using System;
 using System.Collections;
 using System.Xml.Serialization;
-using System.Threading.Tasks;
-using System.Linq;
-using System.ComponentModel;
-using System.Runtime.CompilerServices;
 
 namespace Imendio.Blam
 {
     [XmlType("group")]
-    public class ChannelGroup : IChannel, INotifyPropertyChanged
+    public class ChannelGroup : IChannel
     {
         [XmlAttribute("Name")] public string Int_Name = null;
         [XmlAttribute("Url")] public string Int_Url = null;
         [XmlElement("Channel", typeof(Channel))] public ArrayList Channels;
         private Gtk.TreeIter mIter;
 
-		public event PropertyChangedEventHandler PropertyChanged;
+        public event ChannelEventHandler Updated;
 
         public ArrayList ItemList {
             get { /* FIXME: Cache this value. */
@@ -133,24 +129,10 @@ namespace Imendio.Blam
                 Channels = new ArrayList();
         }
 
-		void ChannelChanged(object sender, PropertyChangedEventArgs args)
-		{
-			// Whatever's changed in the children has changed for us as well
-			NotifyPropertyChanged(args.PropertyName);
-		}
-
-		void NotifyPropertyChanged([CallerMemberName] string name = "")
-		{
-			if (PropertyChanged != null) {
-				PropertyChanged(this, new PropertyChangedEventArgs(name));
-			}
-		}
-
         public void Setup()
         {
             foreach(Channel chan in Channels){
                 chan.Setup();
-				chan.PropertyChanged += ChannelChanged;
             }
         }
 
@@ -175,13 +157,5 @@ namespace Imendio.Blam
             }
             return null;
         }
-
-		public async Task<bool> RefreshAsync()
-		{
-			var tasks = Channels.Cast<Channel>().Select(c => c.RefreshAsync());
-			// FIXME: The continuation is a horrible hack to fit the interface
-			await Task.WhenAll(tasks);
-			return true;
-		}
     }
 }
