@@ -10,14 +10,12 @@ using Gtk;
 using GtkSharp;
 using System;
 using System.Collections;
-using System.Collections.Specialized;
 using System.Runtime.InteropServices;
-using System.ComponentModel;
 using Mono.Unix;
 
 namespace Imendio.Blam {
 
-	public class ChannelList : Gtk.TreeView {
+    public class ChannelList : Gtk.TreeView {
 
         // Events
         public event ChannelEventHandler ChannelSelectedEvent;
@@ -34,101 +32,70 @@ namespace Imendio.Blam {
 
         private TreeViewColumn nameColumn;
 
+        private IEnumerator channelEnumerator;
+        private IEnumerator groupEnumerator;
+
+        private IChannel LastChannel = null;
+
         public static TargetEntry[] DragEntries = new TargetEntry[] {
             new TargetEntry("channel", TargetFlags.Widget, (uint)TargetType.Channel)
        };
 
-		ChannelCollection Collection;
+        public ChannelList(IList channels, IList groups)
+        { 
+            TreeViewColumn col;
+            CellRenderer   cell;
 
-		public ChannelList(ChannelCollection collection)
-		{
-			Collection = collection;
-			Collection.CollectionChanged += CollectionChanged;
-			Collection.PropertyChanged += CollectionPropertyChanged;
+            /* Channel name column */
+            nameColumn = new TreeViewColumn();
+            cell = new CellRendererText();
 
-			TreeViewColumn col;
-			CellRenderer   cell;
+            nameColumn.PackStart(cell, true);
+            nameColumn.Sizing = TreeViewColumnSizing.GrowOnly;
+            nameColumn.Expand = true;
+            nameColumn.SetCellDataFunc(cell, new TreeCellDataFunc(NamesCellDataFunc));
 
-			/* Channel name column */
-			nameColumn = new TreeViewColumn();
-			cell = new CellRendererText();
+            AppendColumn(nameColumn);
 
-			nameColumn.PackStart(cell, true);
-			nameColumn.Sizing = TreeViewColumnSizing.GrowOnly;
-			nameColumn.Expand = true;
-			nameColumn.SetCellDataFunc(cell, new TreeCellDataFunc(NamesCellDataFunc));
+            /* Items column */
+            col = new TreeViewColumn();
+            cell = new CellRendererText();
 
-			AppendColumn(nameColumn);
+            col.PackStart(cell, true);
+            col.SetCellDataFunc(cell,
+                                new TreeCellDataFunc(ItemsCellDataFunc));
 
-			/* Items column */
-			col = new TreeViewColumn();
-			cell = new CellRendererText();
+            AppendColumn(col);
 
-			col.PackStart(cell, true);
-			col.SetCellDataFunc(cell,
-			                    new TreeCellDataFunc(ItemsCellDataFunc));
+            this.RulesHint = true;
 
-			AppendColumn(col);
+            selectionChangedHandler = new EventHandler(SelectionChanged);
+            this.Selection.Changed += selectionChangedHandler;
 
-			this.RulesHint = true;
+            this.Model = new TreeStore (typeof(Channel));
+            this.HeadersVisible = false;
 
-			selectionChangedHandler = new EventHandler(SelectionChanged);
-			this.Selection.Changed += selectionChangedHandler;
+            // Sort the list
+            (Model as TreeStore).DefaultSortFunc = ChannelSort;
+            (this.Model as TreeStore).SetSortColumnId (-1, SortType.Ascending);
 
-			this.Model = new TreeStore (typeof(Channel));
-			this.HeadersVisible = false;
+            // Right click popup
+            this.popupMenu = new ChannelMenu();
+            this.popupMenu.EditSelected       += EditChannelCb;
+            this.popupMenu.MarkAsReadSelected += MarkAsReadCb;
+            this.popupMenu.RemoveSelected     += RemoveChannelCb;
+            this.popupMenu.RefreshSelected    += RefreshChannelCb;
 
-			// Sort the list
-			(Model as TreeStore).DefaultSortFunc = ChannelSort;
-			(this.Model as TreeStore).SetSortColumnId (-1, SortType.Ascending);
-
-			// Right click popup
-			this.popupMenu = new ChannelMenu();
-			this.popupMenu.EditSelected       += EditChannelCb;
-			this.popupMenu.MarkAsReadSelected += MarkAsReadCb;
-			this.popupMenu.RemoveSelected     += RemoveChannelCb;
-			this.popupMenu.RefreshSelected    += RefreshChannelCb;
+            channelEnumerator = channels.GetEnumerator();
+            groupEnumerator = groups.GetEnumerator();
 
 			EnableModelDragSource(ModifierType.Button1Mask, DragEntries, DragAction.Copy);
 			DragDataGet += DragDataGetHandler;
 			EnableModelDragDest(DragEntries, DragAction.Copy);
 			DragDataReceived += DragDataReceivedHandler;
 
-			// Add the groups and chanels to the view
-			foreach (IChannel channel in Collection.Channels)
-				Add (channel);
-
-			foreach (IChannel group in Collection.Groups)
-				AddGroup(group);
-		}
-
-		void CollectionPropertyChanged(object sender, PropertyChangedEventArgs args)
-		{
-			Application.Context.Post(state => Application.TheApp.UpdateTotalNumberOfUnread(), null);
-		}
-
-		void CollectionChanged(object sender, NotifyCollectionChangedEventArgs args)
-		{
-			switch (args.Action) {
-				case NotifyCollectionChangedAction.Add:
-					foreach (var channel in args.NewItems) {
-						if (channel is Channel) {
-							Add((IChannel)channel);
-						} else if (channel is ChannelGroup) {
-							AddGroup((IChannel)channel);
-						} else {
-							dynamic obj = channel;
-							AddToGroup(obj.Group, obj.Channel);
-						}
-					}
-					break;
-				case NotifyCollectionChangedAction.Remove:
-					foreach (IChannel channel in args.OldItems) {
-						Remove(channel);
-					}
-					break;
-			}
-		}
+            GLib.Idle.Add(new GLib.IdleHandler(IdleAdd));
+        }
 
         private void ForceResort()
         {
@@ -143,7 +110,7 @@ namespace Imendio.Blam {
 
 		}
 
-		async void DragDataReceivedHandler(object o, DragDataReceivedArgs args)
+		private void DragDataReceivedHandler(object o, DragDataReceivedArgs args)
 		{
 			TreePath path;
 			TreeViewDropPosition pos;
@@ -158,78 +125,91 @@ namespace Imendio.Blam {
 			if (path == null)
 				return;
 
-			/* Not directly over a channel or group. */
+			/* Not directly over a channel. */
 			if(pos == TreeViewDropPosition.After || pos == TreeViewDropPosition.Before){
-				var src_path = Model.GetPath(src.Iter).Copy();
-				// both at the top level, don't do anything
-				if (path.Depth == 1 && src_path.Depth == 1)
-					return;
-
-				// target depth 1, source >1 means we need to lift the channel out of the group
-				if (path.Depth == 1 && src_path.Depth > 1) {
-					Collection.Move(src, null);
+				TreePath tmp_path = path.Copy();
+				tmp_path.Up();
+				TreePath tmp_path_src = Model.GetPath(src.Iter).Copy();
+				tmp_path_src.Up();
+				if(tmp_path == tmp_path_src){ // Same parent.
+					System.Console.WriteLine("return!");
 					return;
 				}
 
-				// the depth of both is higher than 1, so both are in a group, and this is a channel
-				var src_parent_path = src_path.Copy();
-				src_parent_path.Up();
-				var trg_parent_path = path.Copy();
-				trg_parent_path.Up();
+				Blam.Application.TheApp.CCollection.Remove(src);
 
-				// if they have the same parent, we don't need to do anything
-				if (src_parent_path.Equals(trg_parent_path))
-					return;
-
-				// now we know that we want to move the channel into this channel's group
-				TreeIter parent_iter;
-				Model.GetIter(out parent_iter, trg_parent_path);
-				ChannelGroup parent = (ChannelGroup)Model.GetValue(parent_iter, 0);
-				Collection.Move(src, parent);
+				Model.GetIter(out tmp_iter, tmp_path);
+				IChannel drop_parent = Model.GetValue(tmp_iter, 0) as IChannel;
+				//System.Console.WriteLine("dp: {0} depth:{1}", drop_parent.GetType(), path.Depth);
+				if(path.Depth == 2 && drop_parent is ChannelGroup){ // Dropped inside a group.
+					Blam.Application.TheApp.CCollection.Add(drop_parent as ChannelGroup, src);
+				} else{
+					Blam.Application.TheApp.CCollection.Add(src);
+				}
+			/* Directly over a channel. */
 			} else {
-				// on top of a group, move it here
-				if (dst is ChannelGroup) {
-					// if it's already in this group, don't do anything
-					var grp = (ChannelGroup)dst;
-					if (grp.Channels.Contains(src))
-						return;
+				TreePath tmp_path_src = Model.GetPath(src.Iter).Copy();
+				tmp_path_src.Up();
+				TreePath dst_path_parent = path.Copy();
+				dst_path_parent.Up();
 
-					Collection.Move(src, dst);
-					return;
+				if(dst is Channel && path.Depth == 1 && // dst is top-level
+				   src is Channel && Model.GetPath(src.Iter).Depth == 2){ // src is in a group
+					Blam.Application.TheApp.CCollection.Remove(src);
+					Blam.Application.TheApp.CCollection.Add(src);
 				}
-				// on top of a channel
-				var src_path = Model.GetPath(src.Iter).Copy();
-				if (path.Depth == 1 && src_path.Depth == 1)
-					return;
-				if (path.Depth == 1 && src_path.Depth > 1) {
-					Collection.Move(src, null);
+
+				/*
+				 * A channel can't become a group and a group can't be
+				 * inside another group and it doesn't make sense to move
+				 * it into the same group.
+				 * FIXME: Make it possible for a group to be inside another group.
+				 */
+				if((dst is Channel && path.Depth == 1) || src is ChannelGroup ||
+				   path == tmp_path_src){ // Move to the same group
 					return;
 				}
 
-				// dropped on a channel inside a group
-				var dst_parent_path = path.Copy();
-				dst_parent_path.Up();
+				Blam.Application.TheApp.CCollection.Remove(src);
+				/*
+				 * If we drop on a channel which is inside a group, we add that channel to
+				 * the group.
+				 */
 
-				var src_parent_path = src_path.Copy();
-				src_parent_path.Up();
-				// already in the same group
-				if (src_parent_path.Equals(dst_parent_path))
-					return;
+				if(dst is Channel && path.Depth == 2){ // Channel inside a group.
+					Model.GetIter(out tmp_iter, dst_path_parent);
+					dst = Model.GetValue(tmp_iter, 0) as ChannelGroup;
+				}
 
-				Collection.Move(src, dst);
+				Blam.Application.TheApp.CCollection.Add(dst as ChannelGroup, src);
 			}
 		}
 
+        private bool IdleAdd()
+        {
+            while (channelEnumerator.MoveNext ()) {
+                IChannel channel = channelEnumerator.Current as IChannel;
+                Add(channel);
+            }
+
+            while(groupEnumerator.MoveNext()){
+                IChannel group = groupEnumerator.Current as IChannel;
+                AddGroup(group);
+            }
+
+            return false;
+        }
+
         public void Add (IChannel channel)
         {
             channel.Iter = (this.Model as TreeStore).AppendValues(channel);
-			channel.PropertyChanged += ChannelChanged;
+            channel.Updated += Updated;
         }
 
 		public void AddToGroup(IChannel group, IChannel channel)
 		{
 			channel.Iter = (Model as TreeStore).AppendValues(group.Iter, channel);
-			channel.PropertyChanged += ChannelChanged;
+            channel.Updated += Updated;
 		}
 
         public void AddGroup(IChannel channel)
@@ -257,19 +237,17 @@ namespace Imendio.Blam {
             return(IChannel)model.GetValue(iter, 0);
         }
 
-		void ChannelChanged(object sender, PropertyChangedEventArgs args)
-		{
-			var channel = (IChannel)sender;
+        public void Updated (IChannel channel)
+        {
+            TreeIter iter = FindChannel (channel);
 
-			TreeIter iter = FindChannel (channel);
+            if (!iter.Equals(TreeIter.Zero)) {
+                this.Model.EmitRowChanged (this.Model.GetPath(iter), iter);
+            }
 
-			if (!iter.Equals(TreeIter.Zero)) {
-				Application.Context.Post((state) =>  this.Model.EmitRowChanged (this.Model.GetPath(iter), iter), null);
-			}
+            ForceResort();
 
-			ForceResort();
-
-		}
+        }
 
         public void Remove (IChannel channel)
         {
@@ -280,6 +258,7 @@ namespace Imendio.Blam {
                 this.Selection.Changed -= selectionChangedHandler;
 
                 valid = (this.Model as TreeStore).Remove(ref iter);
+                channel.RemoveItems();
 
                 this.Selection.Changed += selectionChangedHandler;
 
@@ -454,6 +433,8 @@ namespace Imendio.Blam {
                 }
             }*/
 
+            LastChannel = channel;
+
             if (ChannelSelectedEvent != null) {
                 ChannelSelectedEvent(channel);
             }
