@@ -12,63 +12,38 @@ using GLib;
 using GtkSharp;
 using System;
 using System.Collections;
-using System.Threading;
-using System.Threading.Tasks;
 using System.ServiceModel.Syndication;
-using System.ComponentModel;
-using System.Runtime.CompilerServices;
 
 namespace Imendio.Blam {
     
-	public class ItemList : Gtk.TreeView, INotifyPropertyChanged {
+    public class ItemList : Gtk.TreeView {
 
-		public event PropertyChangedEventHandler PropertyChanged;
+	public delegate void ItemSelectedHandler(Imendio.Blam.Item item);
+	public event ItemSelectedHandler ItemSelected;
 
 	private ItemView itemView;
 	
     private TreeViewColumn titleColumn;
     private TreeViewColumn iconColumn;
-    CancellationTokenSource MarkReadCancel;
 
-		IChannel _CurrentChannel;
-		public IChannel CurrentChannel {
-			get {
-				return _CurrentChannel;
-			}
-			set {
-				if (_CurrentChannel != value) {
-					_CurrentChannel = value;
-					UpdateList();
-					NotifyPropertyChanged();
-				}
-			}
-		}
+    private Item lastItem;
+    private uint lastTimeout;
 
-		Item _Current;
-		public Item Current {
-			get {
-				return _Current;
-			}
-			set {
-				if (_Current != value) {
-					_Current = value;
-					NotifyPropertyChanged();
-				}
-			}
-		}
+	private IChannel channel;
+	public IChannel CurrentChannel {
+	    get {
+		return channel;
+	    }
 
-
-		void NotifyPropertyChanged([CallerMemberName] string name = "")
-		{
-			if (PropertyChanged != null) {
-				PropertyChanged(this, new PropertyChangedEventArgs(name));
-			}
-		}
-
-	public ItemList(ItemView itemView, ChannelList channelList)
+	    set {
+		channel = value;
+		UpdateList();
+	    }
+	}
+	
+	public ItemList(ItemView itemView) 
 	{
 	    this.itemView = itemView;
-			channelList.ChannelSelectedEvent += channel => CurrentChannel = channel;
 
         CellRendererPixbuf cell2 = new CellRendererPixbuf();
         iconColumn = new TreeViewColumn();
@@ -99,73 +74,114 @@ namespace Imendio.Blam {
 	    Conf.AddNotify (Conf.GetFullKey(Preference.REVERSE_ENTRIES), new NotifyEventHandler (ConfNotifyHandler));
 
 	}
+	
+	public void ItemAdded (Imendio.Blam.Item item)
+	{/*
+	    if (channel != null && channel == item.Channel) {
+		((ListStore)this.Model).AppendValues(item);
+	    }*/
+	}
 
-		bool Selected(out TreeModel model, out TreeIter iter)
-		{
-			return Selection.GetSelected(out model, out iter);
-		}
-
-		void ScrollTo(TreeIter iter)
-		{
-			GrabFocus();
-			ScrollToCell(this.Model.GetPath(iter), titleColumn, false, 0, 0);
-			SetCursor(this.Model.GetPath(iter), titleColumn, false);
-		}
-
-		bool LoopItems(TreeModel model, ref TreeIter iter, Func<Item, bool> fn)
-		{
-			var start = iter;
-			do {
-				Item item = (Item)model.GetValue(iter, 0);
-				if (item != null && fn(item))
-					return true;
-
-				if (!Model.IterNext(ref iter))
-					Model.GetIterFirst(out iter);
-			} while (!iter.Equals(start));
-
+	public bool NextUnread()
+	{
+	    TreeModel model;
+	    TreeIter  iter;
+	    
+	    if (this.Selection.GetSelected(out model, out iter)) { 
+		if (!this.Model.IterNext(ref iter)) {
+		    if (!((ListStore)this.Model).GetIterFirst(out iter)) {
 			return false;
+		    }
 		}
+	    } else {
+		if (!((ListStore)this.Model).GetIterFirst(out iter)) {
+		    return false;
+		}
+	    }
 
-		public bool Next(bool unread = true)
-		{
-			TreeModel model;
-			TreeIter  iter;
+	    TreeIter startIter = iter;
 
-			// If there's something selected (and it's not the last one),
-			// start looking from there. Otherwise from the start.
-			if (!Selected(out model, out iter) || !Model.IterNext(ref iter))
-				((ListStore)Model).GetIterFirst(out iter);
+	    do {
+		Imendio.Blam.Item item = (Imendio.Blam.Item)model.GetValue(iter, 0);
+		if (item.Unread) {
+		    this.GrabFocus();
+		    ScrollToCell(this.Model.GetPath(iter), titleColumn,
+				 false, 0, 0);
+		    
+		    this.SetCursor(this.Model.GetPath(iter), 
+				   titleColumn, false);
+		    return true;
+		}
+		
+		if (!this.Model.IterNext(ref iter)) {
+		    this.Model.GetIterFirst(out iter);
+		}
+	    } while (!iter.Equals(startIter));
 
-			Func<Item, bool> check;
-			if (unread)
-				check = item => item.Unread;
-			else
-				check = (_) => true;
+	    return false;
+	}
 
-			if (LoopItems(model, ref iter, check)) {
-				ScrollTo(iter);
-				return true;
-			}
+	public bool IdleScrollCb ()
+	{
+	    TreeIter iter;
+	    TreeIter lastIter = TreeIter.Zero;
+	    TreePath path = null;
 
+	    if (!this.Model.GetIterFirst (out iter)) {
+		// No items 
+		return false;
+	    }
+
+	    bool reverse = Conf.Get (Preference.REVERSE_ENTRIES, false);
+	    
+	    do {
+		Item item = (Item) this.Model.GetValue (iter, 0);
+		if (item.Unread) {
+		    if (((ListStore)this.Model).IterIsValid (lastIter)) {
+			path = this.Model.GetPath (lastIter);
+		    } else {
+			path = this.Model.GetPath (iter);
+		    }
+		    break;
+		} 
+		lastIter = iter;
+	    } while (this.Model.IterNext (ref iter));
+	   
+	    if (path == null) { 
+		if (reverse) {
+		    path = TreePath.NewFirst ();
+		} else {
+		    if (!((ListStore)this.Model).IterIsValid (lastIter)) {
 			return false;
+		    }
+		
+		    path = this.Model.GetPath (lastIter);
 		}
+	    }
 
-		public void UpdateList()
-		{
-			if (CurrentChannel == null)
-				return;
+	    if (path != null) {
+		ScrollToCell (path, titleColumn, true, 0, 0);
+	    }
+	    
+	    return false;
+	}
 
-			var list = Model as ListStore;
+	public void UpdateList ()
+	{
+	    ((ListStore)this.Model).Clear();
+            ItemStore store = ItemStore.GetInstance();
+	    if (this.channel == null) {
+		return;
+	    }
+            foreach(string id in channel.ItemList){
+                if(id == null){
+                    continue;
+                }
+                (Model as ListStore).AppendValues(store.Get(id));
+            }
 
-			list.Clear();
-			foreach(string id in CurrentChannel.ItemList){
-				if (id == null)
-					continue;
-
-				list.AppendValues(ItemStore.Get(id));
-			}
-		}
+	    GLib.Timeout.Add (100, new GLib.TimeoutHandler (IdleScrollCb));
+	}
 
 	public void Update (Item item)
 	{
@@ -176,63 +192,54 @@ namespace Imendio.Blam {
 	    }
 	}
 
-		public Item GetSelected()
-		{
-			TreeIter iter;
-			TreeModel model;
+	public Item GetSelected ()
+	{
+	    TreeIter  iter;
+	    TreeModel model;
 
-			if (Selected(out model, out iter))
-				return (Item)model.GetValue(iter, 0);
+	    if (!this.Selection.GetSelected (out model, out iter)) {
+		return null;
+	    }
 
-			return null;
-		}
+	    return (Item) model.GetValue (iter, 0);
+	}
+	
+	private void SelectionChanged (object obj, EventArgs args)
+	{
+	    TreeSelection selection = (TreeSelection) obj;
+	    TreeIter iter;
+	    TreeModel model;
+	    Imendio.Blam.Item item;
 
-		public void EmitSelectedRowChanged()
-		{
-			TreeIter iter;
-			TreeModel model;
+	    if (!selection.GetSelected (out model, out iter)) {
+		return;
+	    }
 
-			if (!Selected(out model, out iter))
-				return;
+	    item = (Imendio.Blam.Item) model.GetValue (iter, 0);
+	    if (item != null) {
+            EmitItemSelected (item);
 
-			model.EmitRowChanged(model.GetPath(iter), iter);
-		}
+            bool useTimeout = (bool) Conf.Get(Preference.MARK_ITEMS_READ, false);
+            if(useTimeout){
+                if(lastTimeout > 0)
+                    GLib.Source.Remove(lastTimeout);
 
-		private async void SelectionChanged (object obj, EventArgs args)
-		{
-			TreeSelection selection = (TreeSelection) obj;
-			Imendio.Blam.Item item;
+                uint readTimeout = (uint) Conf.Get(Preference.MARK_ITEMS_READ_TIMEOUT, 3000);
+                lastItem = item;
+                lastTimeout = GLib.Timeout.Add(readTimeout, new GLib.TimeoutHandler (SetToRead));
+            } else {
+                item.SetUnread(false);
+            }
+	    }
+	}
 
-			item = selection.Item();
-			if (item == null)
-				return;
+    private bool SetToRead()
+    {
+        if(lastItem == GetSelected())
+                lastItem.SetUnread(false);
 
-			if (MarkReadCancel != null)
-				MarkReadCancel.Cancel();
-
-			Current = item;
-
-			bool useTimeout = (bool) Conf.Get(Preference.MARK_ITEMS_READ, false);
-
-			if (!useTimeout) {
-				item.Unread = false;
-				selection.EmitRowChanged();
-				return;
-			}
-
-			MarkReadCancel = new CancellationTokenSource();
-			var readTimeout = (uint) Conf.Get(Preference.MARK_ITEMS_READ_TIMEOUT, 3000);
-
-			try {
-				await Task.Delay(TimeSpan.FromMilliseconds(readTimeout), MarkReadCancel.Token);
-			} catch (OperationCanceledException) {
-				return;
-			}
-
-			MarkReadCancel = null;
-			item.Unread = false;
-			selection.EmitRowChanged();
-		}
+        return false;
+    }
 
 		private int CompareFunc(TreeModel model, TreeIter a, TreeIter b)
 		{
@@ -272,8 +279,7 @@ namespace Imendio.Blam {
 		weight = (int) Pango.Weight.Bold;
 	    }
 
-
-        ((CellRendererText)cell).Text = HtmlUtils.Unescape((item.Title as TextSyndicationContent).Text.Trim());
+        ((CellRendererText)cell).Text = (item.Title as TextSyndicationContent).Text.Trim();
         ((CellRendererText)cell).Weight = weight;
         ((CellRendererText)cell).Ellipsize = Pango.EllipsizeMode.End;
 	}
@@ -301,6 +307,13 @@ namespace Imendio.Blam {
         }
     }
 	
+	private void EmitItemSelected (Imendio.Blam.Item item) 
+	{
+	    if (ItemSelected != null) {
+		ItemSelected (item);
+	    }
+	}
+	
 	private void SetSortOrder (bool reverseEntries)
 	{
 	    SortType sortType = SortType.Ascending;
