@@ -13,12 +13,10 @@ using GtkSharp;
 using WebKit;
 using Mono.Unix;
 using System.Collections;
-using System.Collections.Generic;
 using System.Reflection;
 using System.IO;
 using System.Net;
 using System.Text;
-using System.ComponentModel;
 using System.ServiceModel.Syndication;
 
 namespace Imendio.Blam {
@@ -27,26 +25,23 @@ namespace Imendio.Blam {
 #if ENABLE_FONTS
         private WebSettings webSettings;
 #endif
+	private Imendio.Blam.Item currentItem;
     private string baseDir = null;
 	private string last_link = null;
 	public bool PageLoaded;
 
         public event StringUpdatedHandler OnUrl;
 
-		Item _Current;
-		public Item Current {
-			get {
-				return _Current;
-			}
-			set {
-				if (_Current != value) {
-					_Current = value;
-					Load();
-				}
-			}
-		}
-
-		ItemList itemList;
+	public Imendio.Blam.Item CurrentItem {
+	    get {
+		return currentItem;
+	    }
+	     
+	    set {
+		currentItem = value;
+		Load ();
+	    }
+	}
 
         public WebView Widget {
             get {
@@ -54,10 +49,8 @@ namespace Imendio.Blam {
             }
         }
 
-		public ItemView(ItemList list) : base()
-		{
-            itemList = list;
-			itemList.PropertyChanged += ItemListPropertyChanged;
+	public ItemView () : base()
+        {
             this.webView = new WebView ();
 #if ENABLE_FONTS
             this.webSettings = new WebSettings ();
@@ -81,8 +74,8 @@ namespace Imendio.Blam {
 			 * (as we know he was hovering). Thus, try to open it on the browser
 			 */
 			if(args.Request.Uri.Equals(last_link)){
-				args.RetVal = NavigationResponse.Ignore;
 				GtkBeans.Global.ShowUri(null, args.Request.Uri);
+				args.RetVal = NavigationResponse.Ignore;
 			} else {
 				/* Otherwise, it's part of the post, so accept it (like the facebook iframe) */
 				args.RetVal = NavigationResponse.Accept;
@@ -102,46 +95,38 @@ namespace Imendio.Blam {
             webView.Show ();
             PageLoaded = false;
         }
-
-		void ItemListPropertyChanged (object sender, PropertyChangedEventArgs e)
-		{
-			if (e.PropertyName.Equals("Current"))
-				Current = ((ItemList)sender).Current;
-		}
-
+        
         private void Load()
         {
 			Theme theme = Application.TheApp.ThemeManager.CurrentTheme;
             string author;
-            if(Current.Authors.Count == 0){
+            if(currentItem.Authors.Count == 0){
                 author = string.Empty;
-            } else if(Current.Authors[0].Name != null){
-                author = String.Format(Catalog.GetString("by {0}"), Current.Authors[0].Name);
+            } else if(currentItem.Authors[0].Name != null){
+                author = String.Format(Catalog.GetString("by {0}"), currentItem.Authors[0].Name);
             } else {
                 author  = "&nbsp;";
             }
             string link    = Catalog.GetString("Show in browser");
-            string pubdate = (!Current.LastUpdatedTime.Equals(DateTimeOffset.MinValue)) ?
-			                    Current.LastUpdatedTime.ToString("D", System.Globalization.CultureInfo.CurrentUICulture) :
-				(!Current.PublishDate.Equals(DateTimeOffset.MinValue)) ?
-                  Current.PublishDate.ToString("D", System.Globalization.CultureInfo.CurrentUICulture) :
+            string pubdate = (!currentItem.LastUpdatedTime.Equals(DateTimeOffset.MinValue)) ?
+			                    currentItem.LastUpdatedTime.ToString("D", System.Globalization.CultureInfo.CurrentUICulture) :
+				(!currentItem.PublishDate.Equals(DateTimeOffset.MinValue)) ?
+                  currentItem.PublishDate.ToString("D", System.Globalization.CultureInfo.CurrentUICulture) :
 				  Catalog.GetString("[No date available]");
-            string text = Current.Summary != null ? Current.Summary.Text
-                : Current.Content != null ? (Current.Content as TextSyndicationContent).Text : null;
-			string title = Current.Title.Text;
-            string url = Current.Links.Count == 0 ? Current.Id :
-                Current.Links[0].Uri.ToString();
-
+            string text = currentItem.Summary != null ? currentItem.Summary.Text
+                : currentItem.Content != null ? (currentItem.Content as TextSyndicationContent).Text : null;
+            string title   = HtmlUtils.Escape(currentItem.Title.Text);
+            string url     = currentItem.Links[0].Uri.ToString();
             baseDir        = "file://" + theme.Path;
 
-            var replaces = new Dictionary<string, string> {
-                {"author", author},
-                {"link", link},
-                {"pubdate", pubdate},
-                {"text", text},
-                {"title", title},
-                {"url", url},
-                {"localbase", baseDir},
+            string[] replaces = {
+                "author", author,
+                "link", link,
+                "pubdate", pubdate,
+                "text", text,
+                "title", title,
+                "url", url,
+                "localbase", baseDir
             };
 
 			webView.LoadString(theme.Render(replaces), null, null, baseDir);
